function [Strain,Tilt]=pCDM_StrainTiltSurf(X,Y,X0,Y0,depth,...
    omegaX,omegaY,omegaZ,DVx,DVy,DVz,nu)
% pCDM_StrainTiltSurf
% calculates the surface strains and tilts caused by a point CDM that is 
% composed of three mutually orthogonal point tensile dislocations in a
% uniform elastic half-space.
% 
% pCDM: point Compound Dislocation Model
% PTD: Point Tensile Dislocation
% EFCS: Earth-Fixed Coordinate System
%
% INPUTS
% X, Y and Z:
% Coordinates of calculation points in EFCS (East, North, Up). The matrices
% X and Y must have the same size.
%
% X0, Y0 and depth:
% Horizontal coordinates (in EFCS) and depth of the point CDM. The depth
% must be a positive value. X0, Y0 and depth have the same unit as X, Y and
% Z.
%
% omegaX, omegaY and omegaZ:
% Clockwise rotation angles about X, Y and Z axes, respectively, that 
% specify the orientation of the point CDM in space. These angles must be
% given in degrees.
%
% DVx, DVy and DVz:
% Potencies of the PTDs. Before applying the rotations the PTDs are normal  
% to the X, Y and Z axes, respectively. The potency has the unit of volume 
% (the unit of displacements and CDM semi-axes to the power of 3).
%
% mu and lambda:
% Lam constants.
% 
% 
% OUTPUTS
% Strain:
% Calculated strain tensor components in EFCS. The four columns of Strain
% are Exx, Eyy, Ezz, Exy, respectively. Note that at the free surface 
% (z = 0) Exz and Eyz are zero. The strain components are dimensionless.
%
% Tilt:
% Calculated tilt components in EFCS. The two columns of Tilt are x 
% component (duz_dx) and the y component (duz_dy), respectively. Tilts are 
% calculated in radians.
%
% Example: Calculate and plot strain and tilt components along the X axis
%
% [X,Y] = meshgrid(0:50:7000,0);
% X0 = 0; Y0 = 0; depth = 3100; omegaX = 8; omegaY = -5; omegaZ = -331;
% DVx = 0.7e6; DVy = 0.8e6; DVz = 0.5e6; nu = 0.25;
% [Strain,Tilt] = pCDM_StrainTiltSurf(X,Y,X0,Y0,depth,...
%     omegaX,omegaY,omegaZ,DVx,DVy,DVz,nu);
% figure
% subplot(211)
% title('Strain components')
% plot(X,Strain(:,1)*1e6,'r')
% hold on
% plot(X,Strain(:,2)*1e6,'g')
% plot(X,Strain(:,3)*1e6,'b')
% plot(X,Strain(:,4)*1e6,'k')
% % xlabel('X (meter)')
% ylabel('\epsilon_{ij}\times{10^6}')
% legend('\epsilon_{xx}','\epsilon_{yy}','\epsilon_{zz}','\epsilon_{xy}')
% subplot(212)
% title('Tilt components')
% plot(X,Tilt(:,1)*1e6,'r')
% hold on
% plot(X,Tilt(:,2)*1e6,'g')
% xlabel('X (meter)')
% ylabel('\theta_{i}\times{10^6} radians')
% legend('\theta_x','\theta_y')

% Reference journal articles:
% 1)
% Nikkhoo, M., Walter, T. R., Lundgren, P. R., Prats-Iraola, P. (2017):
% Compound dislocation models (CDMs) for volcano deformation analyses.
% Geophysical Journal International, doi: 10.1093/gji/ggw427
%
% 2)
% Nikkhoo, M., Rivalta, E. (2022b):
% Surface deformations and gravity changes caused by pressurized finite 
% ellipsoidal cavities. Geophysical Journal International, doi: 
% 10.1093/gji/ggac351

% Copyright (c) 2022 Mehdi Nikkhoo
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files
% (the "Software"), to deal in the Software without restriction, including
% without limitation the rights to use, copy, modify, merge, publish,
% distribute, sublicense, and/or sell copies of the Software, and to permit
% persons to whom the Software is furnished to do so, subject to the
% following conditions:
%
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
% MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
% NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
% DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
% OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
% USE OR OTHER DEALINGS IN THE SOFTWARE.

% I appreciate any comments or bug reports.

% Mehdi Nikkhoo
% Created: 2015.5.22
% Last modified: 2022.9.2
%
% Section 2.1, Physics of Earthquakes and Volcanoes
% Department 2, Geophysics
% Helmholtz Centre Potsdam
% German Research Centre for Geosciences (GFZ)
%
% email:
% mehdi.nikkhoo@gfz-potsdam.de
% mehdi.nikkhoo@gmail.com
%
% website:
% http://www.volcanodeformation.com

% The potencies must have the same sign
DVsign = [sign(DVx) sign(DVy) sign(DVz)];
if any(DVsign>0) && any(DVsign<0)
    error('Input error: DVx, DVy and DVz must have the same sign!')
end

X = X(:);
Y = Y(:);

Rx = [1 0 0;0 cosd(omegaX) sind(omegaX);0 -sind(omegaX) cosd(omegaX)];
Ry = [cosd(omegaY) 0 -sind(omegaY);0 1 0;sind(omegaY) 0 cosd(omegaY)];
Rz = [cosd(omegaZ) sind(omegaZ) 0;-sind(omegaZ) cosd(omegaZ) 0;0 0 1];
R = Rz*Ry*Rx;

Vstrike1 = [-R(2,1),R(1,1),0];
Vstrike1 = Vstrike1/norm(Vstrike1);
strike1 = atan2(Vstrike1(1),Vstrike1(2))*180/pi;
if isnan(strike1)
    strike1 = 0;
end
dip1 = acosd(R(3,1));

Vstrike2 = [-R(2,2),R(1,2),0];
Vstrike2 = Vstrike2/norm(Vstrike2);
strike2 = atan2(Vstrike2(1),Vstrike2(2))*180/pi;
if isnan(strike2)
    strike2 = 0;
end
dip2 = acosd(R(3,2));

Vstrike3 = [-R(2,3),R(1,3),0];
Vstrike3 = Vstrike3/norm(Vstrike3);
strike3 = atan2(Vstrike3(1),Vstrike3(2))*180/pi;
if isnan(strike3)
    strike3 = 0;
end
dip3 = acosd(R(3,3));

% Calculate contribution of the first PTD
if DVx~=0
    [Str1,Tlt1] = PTD_StrainTiltSurf(X,Y,X0,Y0,depth,strike1,dip1,DVx,nu);
else
    Str1 = zeros(numel(X),4);
    Tlt1 = zeros(numel(X),2);
end

% Calculate contribution of the second PTD
if DVy~=0
    [Str2,Tlt2] = PTD_StrainTiltSurf(X,Y,X0,Y0,depth,strike2,dip2,DVy,nu);
else
    Str2 = zeros(numel(X),4);
    Tlt2 = zeros(numel(X),2);
end

% Calculate contribution of the third PTD
if DVz~=0
    [Str3,Tlt3] = PTD_StrainTiltSurf(X,Y,X0,Y0,depth,strike3,dip3,DVz,nu);
else
    Str3 = zeros(numel(X),4);
    Tlt3 = zeros(numel(X),2);
end

Strain = Str1+Str2+Str3;
Tilt = Tlt1+Tlt2+Tlt3;


function [Strain,Tilt]=PTD_StrainTiltSurf(X,Y,X0,Y0,depth,strike,dip,DV,nu)
% PTD_StrainTiltSurf
% calculates the surface strains and tilts due to a point tensile 
% dislocation (PTD) in a uniform elastic half-space (Okada, 1992).

% Note: For a PTD M0 = DV*mu!

x = X(:)-X0;
y = Y(:)-Y0;

beta = strike-90;
Rz = [cosd(beta) -sind(beta);sind(beta) cosd(beta)];
r_beta = Rz*[x y]';
x = r_beta(1,:)';
y = r_beta(2,:)';

d = depth;
r = (x.^2+y.^2+d.^2).^0.5;
q = y*sind(dip)-d*cosd(dip);

J1 = (1-2*nu)*(-3*x.*y.*(3*r+d)./r.^3./(r+d).^3+...
    3*x.^3.*y.*(5*r.^2+4*r*d+d^2)./r.^5./(r+d).^4);
J2 = (1-2*nu)*(1./r.^3-3./r./(r+d).^2+...
    3*x.^2.*y.^2.*(5*r.^2+4*r*d+d^2)./r.^5./(r+d).^4);
J3 = (1-2*nu)*(1./r.^3-3*x.^2./r.^5)-J2;

K1 = -(1-2*nu)*y.*((2*r+d)./r.^3./(r+d).^2-...
    x.^2.*(8*r.^2+9*r.*d+3*d^2)./r.^5./(r+d).^3);
K2 = -(1-2*nu)*x.*((2*r+d)./r.^3./(r+d).^2-...
    y.^2.*(8*r.^2+9*r.*d+3*d^2)./r.^5./(r+d).^3);
K3 = -(1-2*nu)*3*x*d./r.^5-K2;

% Calculate the partial derivatives
dux_dx = DV/2/pi*(3*q.^2./r.^5.*(1-5*x.^2./r.^2)-J3*sind(dip)^2);
dux_dy = DV/2/pi*(3*x.*q./r.^5.*(2*sind(dip)-5*y.*q./r.^2)-J1*sind(dip)^2);
duy_dx = DV/2/pi*(-15*x.*y.*q.^2./r.^7-J1*sind(dip)^2);
duy_dy = DV/2/pi*(3*q./r.^5.*(q+2*y*sind(dip)-5*y.^2.*q./r.^2)-...
    J2*sind(dip)^2);
duz_dx = -DV/2/pi*(-15*x*d.*q.^2./r.^7-K3*sind(dip)^2);
duz_dy = -DV/2/pi*(3*d*q./r.^5.*(2*sind(dip)-5*y.*q./r.^2)-K1*sind(dip)^2);
% dux_dz = -duz_dx; % valid only at the free surface!
% duy_dz = -duz_dy; % valid only at the free surface!
duz_dz = -nu/(1-nu)*(dux_dx+duy_dy); % valid only at the free surface!

% Calculate strains in the Okada (1985) coordinate system
exx = dux_dx;
eyy = duy_dy;
ezz = duz_dz;
exy = (dux_dy+duy_dx)/2;
% exz = (dux_dz+duz_dx)/2;
% eyz = (duy_dz+duz_dy)/2;

% calculate strains in EFCS
Exx = cosd(beta)^2*exx+2*cosd(beta)*sind(beta)*exy+sind(beta)^2*eyy;
Eyy = sind(beta)^2*exx-2*sind(beta)*cosd(beta)*exy+cosd(beta)^2*eyy;
Ezz = ezz;
Exy = -cosd(beta)*sind(beta)*exx+(cosd(beta)^2-sind(beta)^2)*exy+...
    cosd(beta)*sind(beta)*eyy;
% Exz = cosd(beta)*exz+sind(beta)*eyz;
% Eyz = -sind(beta)*exz+cosd(beta)*eyz;

% Strain = [Exx,Eyy,Ezz,Exy,Exz,Eyz];
Strain = [Exx,Eyy,Ezz,Exy];

% Calculate tilts in EFCS: [tilt_x tilt_y]
Tilt = [duz_dx duz_dy]*Rz;